struct Storage
{
 u8 pow_state;
 u8 pow_val;
 Rm reg_mode;

 u8 checksum;
 u8 cycle_cnt;
};

Storage const Def_storage =
{
 true,
 6,
 r_wmiss,
 ~(true + 6 + (u8)r_wmiss),
 0
};

Storage const Max_val_storage =
{
 true,
 MAX_POW,
 r_nosinch,
 ~(true + MAX_POW + (u8)r_nosinch),
 0xff
};

__root __eeprom __no_init Storage Ee_storage [0x1ff / sizeof (Storage)];
const u8 CNM = sizeof(Ee_storage)/sizeof(Storage);


bool check_free_ee (const Storage strg)
{
 if((u8)strg.pow_state == strg.checksum) return true;

 return false;
}

u8 seek_free_storage(__eeprom Storage ee_strg[], const u8 n = CNM)
{
 for(u8 i = 0; i < n; i++)
  if(check_free_ee(ee_strg[i])==true) return i;

 return (u8)-1;
}

void erase_storage(void)
{
 const Storage Er_storage = {0xff, 0xff, (Rm)0xff, 0xff, 0};

 for (u8 i = 0; i < CNM; i++) Ee_storage[i] = Er_storage;

}

u8 calc_checksum(Storage *strg)
{
 u8 tmp = 0;

 for(u8 i = 0; i < sizeof(Storage)/sizeof(u8)-2; i++) tmp += *((u8*)strg +i);

 return ~tmp;
}

bool compare_storage(Storage *strg, __eeprom Storage ee_strg[])
{
 if(strg->pow_state != ee_strg->pow_state) return false;
 if(strg->pow_val   != ee_strg->pow_val)   return false;
 if(strg->reg_mode  != ee_strg->reg_mode)  return false;

 return true;
}

//read_storage(&tmp, Ee_storage);
bool read_storage(Storage *strg, __eeprom Storage ee_strg[], const u8 n = CNM)
{
 bool retval = false;
 u8 z = seek_free_storage(ee_strg);

 //    ,     
 if(z==0)             *strg = Def_storage;
 //        
 else if(z == (u8)-1) *strg = ee_strg[n-1];
 //    
 else                 *strg = ee_strg[z-1];

 //  
 if(calc_checksum(strg) == strg->checksum) retval = true;

 return retval;
}

// write_storage(&tmp, Ee_storage);
bool write_storage(Storage *strg, __eeprom Storage ee_strg[], const u8 n = CNM)
{
 bool retval = false;


 u8 z = seek_free_storage(ee_strg);

 //          
 if(z == (u8)-1) z = n-1;
 //      
 else if(z) z--;
 //          
 else ee_strg[z] = Def_storage;

 //       
 //       
 //        
 if(ee_strg[z].cycle_cnt == (u8)-1)
 {
  if(z < n-1) z++;
  else
  {
   erase_storage();
   z = 0;
  }
 }

 //        
 if(compare_storage(strg, &ee_strg[z])==false)
 {
  //   
  ee_strg[z].cycle_cnt++;

  //    ,      
  if(ee_strg[z].pow_state <= Max_val_storage.pow_state)ee_strg[z].pow_state = strg->pow_state;
  else ee_strg[z].pow_state = Max_val_storage.pow_state;

  if(ee_strg[z].pow_val   <= Max_val_storage.pow_val  )ee_strg[z].pow_val   = strg->pow_val  ;
  else ee_strg[z].pow_val = Max_val_storage.pow_val;

  if(ee_strg[z].reg_mode  <= Max_val_storage.reg_mode )ee_strg[z].reg_mode  = strg->reg_mode ;
  else ee_strg[z].reg_mode = Max_val_storage.reg_mode;

  //     
  strg->checksum = calc_checksum(strg);

  ee_strg[z].checksum  = strg->checksum ;
 }

 //  
 if(calc_checksum(strg) == ee_strg[z].checksum) retval = true;

 return retval;
}

