/********************************************************
Workspace: IAR
Chip type: MEGA8
Project  : SI_PREG
Date     : JUN,2021
Version  : 0
*********************************************************/
#include <ioavr.h>
#include <intrinsics.h>
#include <stdbool.h>
#include <stdlib.h>
#include <math.h>

#include <bitset.c>

#include "defines.c"
#include "periph_init.c"
#include "display_work.c"
#include "kb.c"
#include "pow_key_control.c"
#include "beep.c"
#include "eeprom_rtns.c"

#include "interrupts.c"

void devcon();

void main()
{
 cfg.gpio();
 cfg.t0();
 cfg.t1();
 cfg.acmp();
 cfg.iwdg();

 __enable_interrupt();


 m:
 {
  display_data();

  if(TIFR & (1<<TOV0))
  {
   TIFR = 1<<TOV0;

   scan_keys();
   devcon();
   beep();

   __watchdog_reset();
  }
 }
 goto m;
}

void devcon()
{
 static bool itr=0;
 static enum {prestart, start, work, off} dev_st = prestart, dev_st_tmp;
 static u32 cntdn;
 static bool btn_trg;
 static Storage ustanovki;
 static u8 pow;

 if(dev_st != dev_st_tmp)
 {
  itr = 0;
  dev_st_tmp = dev_st;
 }

 if(btn[0] == rls && btn[1] == rls) btn_trg = false;


 switch (dev_st)
 {
  case prestart:
  {
   static u16 cntdn;
   static u8 ust_tmp;

   if(!itr)
   {
    itr = 1;

    pow = (u8)-1;

    if(read_storage(&ustanovki, Ee_storage)==false) ustanovki = Def_storage;

    ust_tmp = (u8)ustanovki.reg_mode;

    cntdn = T_SEC*5;
   }



   if(btn_trg == false)
   {
    if(cntdn)cntdn--;
    else dev_st = start;

    if(btn[0] != rls)
    {
     btn_trg = true;

     if(ust_tmp)ust_tmp--;
     else       ust_tmp = r_nosinch;

     beep(2, T_SEC/4);
    }

    else if(btn[1] != rls)
    {
     btn_trg = true;

     if(ust_tmp < r_nosinch) ust_tmp++;
     else                    ust_tmp = 0;

     beep(4, T_SEC/4);
    }
   }
   else cntdn = T_SEC*5;

   //          
   if(btn_trg) ustanovki.pow_state = true;

   switch((Rm)ust_tmp)
   {
    case r_wmiss  : dbuf = '"'; break;
    case r_front  : dbuf = 'j'; break;
    case r_spad   : dbuf = 'l'; break;
    case r_frspd  : dbuf = 'u'; break;
    case r_nosinch: dbuf = 'o'; break;
    default: break;
   }

   if(dev_st != dev_st_tmp)
   {
    ustanovki.reg_mode = (Rm)ust_tmp;
    write_storage(&ustanovki, Ee_storage);
   }
  }
  break;

  case start:
  {
   static u8 dcnt;

   if(!itr)
   {
    itr=1;

    //      
    if((bool)ustanovki.pow_state == false) cntdn = 0;
    else                                   cntdn = 1*60*T_SEC;

    pow = 0;

    dcnt = 1;
    dpp = 0;
    beep(5, T_SEC/2);
   }

   if(dcnt)dcnt--;
   else
   {
    dcnt = T_SEC/2;
    inwork_animation(&dbuf, 1);

    if(pow < MAX_POW) pow++;
   }


   //        
   if(btn_trg == false)
   {
    if(btn[0] != rls || btn[1] != rls)
    {
     btn_trg = true;
     cntdn = 0;
    }
   }

   if(cntdn) cntdn--;
   else
   {
    if((bool)ustanovki.pow_state == false) dev_st = off;
    else                                   dev_st = work;
   }
  }
  break;

  case work:
  {
   if(!itr)
   {
    itr=1;

    cntdn = CUT_OFF_CNTDN;

    pow = ustanovki.pow_val;

    dpp = 0;
    beep(6, T_SEC);
   }

   //   
   {
    static bool bt0, bt1;

    if(btn_trg == false)
    {
     bt0 = false;
     bt1 = false;

     if(btn[0] != rls)
     {
      btn_trg = true;
      bt0 = true;

      if(pow)
      {
       pow--;
       beep(2, T_SEC/4);
      }
      else beep(2, T_SEC);
     }

     else if(btn[1] != rls)
     {
      btn_trg = true;
      bt1 = true;

      if(pow < MAX_POW)
      {
       pow++;
       beep(4, T_SEC/4);
      }
      else beep(4, T_SEC);
     }
    }
    else cntdn = CUT_OFF_CNTDN;

    if(btn[0]==hold && btn[1]==hold)
    {
     btn_trg = true;

     if(bt0) pow++;
     if(bt1) pow--;

     dev_st = off;
    }
   }

   dbuf = pow;

   cntdn--;
   if(!cntdn) dev_st = off;
  }
  break;

  case off:
  {
   if(!itr)
   {
    itr=1;

    dbuf = ' ';
    dpp  = 1;

    if((bool)ustanovki.pow_state == true)
    {
     ustanovki.pow_val = pow;
     ustanovki.pow_state = false;
     write_storage(&ustanovki, Ee_storage);
    }

    pow = (u8)-1;

    beep(1,T_SEC);
   }

   //    
   if(btn_trg == false)
   {
    if(btn[0] != rls || btn[1] != rls)
    {
     btn_trg = true;

     ustanovki.pow_state = true;
     write_storage(&ustanovki, Ee_storage);

     dev_st = start;
    }
   }
  }
  break;
 }

 relpow(pow, Key_pow, ustanovki.reg_mode, Rmi);
}

